import axios from 'axios'
import oidc from 'oidc-client'

const oidcmgr = new oidc.UserManager({
  userStore: new oidc.WebStorageStateStore(),
  authority: process.env.VUE_APP_API_STEERING_OIDC_URI,
  // the client id has to be a string, therefore we add the + ''
  client_id: process.env.VUE_APP_OIDC_CLIENT_ID,
  redirect_uri: process.env.VUE_APP_API_STEERING_OIDC_REDIRECT_URI,
  silent_redirect_uri: 'http://localhost:8080/oidc_callback_silentRenew.html',
  popup_redirect_uri: 'http://localhost:8080/oidc_callback_popupRenew.html',
  accessTokenExpiringNotificationTime: process.env.VUE_APP_API_STEERING_OIDC_EXPIRE_NOTIFICATION,
  response_type: 'id_token token',
  scope: 'openid profile email',
  post_logout_redirect_uri: process.env.VUE_APP_API_STEERING_OIDC_REDIRECT_URI_POSTLOGOUT,
  loadUserInfo: true
})

const state = {
  items: [],
  user: {
    name: '',
    email: '',
    access_token: '',
    expires_at: 0,
    logged_in: false,
    steeringUser: null
  },
  userOIDC: null,
}

const getters = {
  items: state => state.items,
  itemCount: state => state.items.length,
}

const mutations = {
  addItem (state, item) {
    state.items.push(item)
  },

  setUserProperties (state, user) {
    state.userOIDC = user
    state.user.logged_in = true
    state.user.name = user.profile.nickname
    state.user.email = user.profile.email
    state.user.access_token = user.access_token
  },

  clearUserProperties (state) {
    state.userOIDC = null
    state.user.logged_in = false
    state.user.name = ''
    state.user.email = ''
    state.user.access_token = ''
  },

  setSteeringUser (state, user) {
    state.user.steeringUser = {...user}
  },

  setAccessToken (state, token) {
    state.user.access_token = token
  },

  signOut (state) {
    state.user.logged_in = false
  },

}

const actions = {
  addItem ({commit}, item) {
    commit('addItem', item)
  },

  fetchSteeringUser (ctx) {
    axios.get(process.env.VUE_APP_API_STEERING + 'users/', {
      withCredentials: true,
      headers: { 'Authorization': 'Bearer ' + ctx.state.user.access_token }
    }).then(response => {
      if (response.data.length === 0) {
        alert('No user profile data provided by steering backend!')
      } else if (response.data.length === 1) {
        ctx.commit('setSteeringUser', response.data[0])
      } else {
        // in case we are a superuser, we get all users returned
        // so we have to iterate through the user list to find out own profile
        for (let u in response.data) {
          if (response.data[u].username === ctx.state.user.name) {
            ctx.commit('setSteeringUser', response.data[u])
            break
          }
        }
      }
    }).catch(error => {
      this.$log.error(error.response.status + ' ' + error.response.statusText)
      this.$log.error(error.response)
      alert('Error: could not fetch user object from the steering backend. See console for details.')
    })
  },

  oidcInit (ctx) {
    // TODO: remove oidc logging after thorough testing
    oidc.Log.logger = console
    this.$log.debug('Initializing oidc client')

    oidcmgr.events.addAccessTokenExpiring(() => {
      this.$log.debug('Starting silent access_token renewal')
      oidcmgr.signinSilent().then(user => {
        ctx.commit('setAccessToken', user.access_token)
        this.$log.debug('Access token:', ctx.state.user.access_token)
      }).catch(err => {
        this.$log.error(err)
        alert('Your OpenID access token could not be renewed automatically.\n' +
          'You will be logged out in ' + process.env.VUE_APP_API_STEERING_OIDC_EXPIRE_NOTIFICATION + ' seconds.')
      })
    })

    oidcmgr.events.addAccessTokenExpired(() => {
      // at this point where the app is just mounted, we cannot use the
      // vue logger, because this.$log does not yet exist. therefore we log
      // to the console, but only if the environment is set tu debug log level
      if (process.env.VUE_APP_LOGLEVEL === 'debug') {
        this.$log.debug("OIDC token has expired. Logging out.")
      }
      ctx.commit('signOut')
    })

    oidcmgr.getUser().then(user => {
      if (user == null) {
        ctx.commit('clearUserProperties')
      } else {
        // TODO: check user.expires_at
        // if token already expired try to get a new one or mark the user as logged out
        ctx.commit('setUserProperties', user)
        ctx.dispatch('fetchSteeringUser')
        // TODO: remove debug info after thorough testing
        this.$log.debug(new Date(user.expires_at * 1000).toString())
        this.$log.debug(new Date(user.expires_at * 1000).toUTCString())
        this.$log.debug(user.access_token)
      }
    }).catch(err => {
      this.$log.debug(err)
      alert('Error: could not fetch OIDC user object. See console for details.')
    })

  },

  signinRedirect () {
    oidcmgr.signinRedirect().catch(err => {
      alert('Error: something went wrong when signing in. See console for details.')
      this.$log.error(err)
    })
  },

  signoutRedirect (ctx) {
    oidcmgr.signoutRedirect().then(resp => {
      ctx.commit('signOut')
      this.$log.debug('signed out', resp)
    }).catch(err => {
      this.$log.error(err)
      alert('Error: something went wrong when logging out. See console for details.')
    })
  }
}

export default {
  namespaced: true,
  state,
  getters,
  actions,
  mutations,
}
